#include <Pilot.h>
#include "globals.h"
#include "send.h"
#include "history.h"

/* returns true if it actually could move in the history */
Boolean MoveInHistory(int amt) {  
  UInt cardNo;
  LocalID localID;
  ULong numRecs;
  int ch = curHistory;
  VoidHand URL, tcurURL;
  ULong tcurURLLength;
  void *url;
  void *cururl;

  ch += amt;
  if (ch < 0) ch = 0;
  if (ch >= histSize) ch = histSize-1;
  
  DmOpenDatabaseInfo(historyDB, &localID, NULL, NULL, &cardNo, NULL);
  DmDatabaseSize(cardNo, localID, &numRecs, NULL, NULL);

  if ((numRecs > ch) && (!amt || (ch != curHistory)) ) {
    int sent;
    
    URL = DmGetRecord(historyDB, ch);        
    tcurURLLength = MemHandleSize(URL);
    tcurURL = MemHandleNew(tcurURLLength);
    cururl = MemHandleLock (tcurURL);
    url = MemHandleLock (URL);
    MemMove (cururl, url, tcurURLLength);
    MemHandleUnlock(URL);
    MemHandleUnlock(tcurURL);
    sent = SendURL(&conID, tcurURL, false, true, false, true);
    if (!sent) {
      MemHandleFree(tcurURL);
    } else if (sent == 2) {
      curHistory = ch;
    } else {
      curHistory = ch;
      tcurHistory = ch;
    }
    DmReleaseRecord(historyDB, ch, false);
    return true;
  }
  return false;
}

void PutInHistory(VoidHand URL, Word length) {
  ULong numRecs;
  void * url;
  VoidHand recH;
  void * recP;
  UInt index;
  UInt cardNo;
  LocalID localID;
  Boolean doit;
  
  /* 
  ** we start out with curHistory at -1 so you can do a back right away
  ** and get the last page oyu visited, but we need to handle what
  ** happens if you don't hit the back button
  */
  if (curHistory < 0) curHistory = 0;
  
  doit = true;

  recH = DmQueryRecord(historyDB, curHistory);
  if (recH) {
    recP = MemHandleLock(recH);
    url = MemHandleLock(URL);
    if ((MemHandleSize(recH) == length) && !(MemCmp(recP, url, length))) {
      doit = false;
    }
    MemHandleUnlock(URL);
    MemHandleUnlock(recH);
  }
  
  if (curHistory) {
    recH = DmQueryRecord(historyDB, curHistory-1);
    if (recH) {
      recP = MemHandleLock(recH);
      url = MemHandleLock(URL);
      if ((MemHandleSize(recH) == length) && !(MemCmp(recP, url, length))) {
	doit = false;
	curHistory--;
      }
      MemHandleUnlock(URL);
      MemHandleUnlock(recH);
    }
  }

  if (!doit) {
    return;
  }

  while (curHistory) {
    DmRemoveRecord(historyDB, 0);
    curHistory--;
  }

  index = 0;
  recH = DmNewRecord(historyDB, &index, (ULong) length);
  url = MemHandleLock(URL);  
  recP = MemHandleLock(recH);
  DmWrite(recP, 0, url, length);
  MemHandleUnlock(recH);
  MemHandleUnlock(URL);
  DmReleaseRecord(historyDB, index, true);

  DmOpenDatabaseInfo(historyDB, &localID, NULL, NULL, &cardNo, NULL);
  DmDatabaseSize(cardNo, localID, &numRecs, NULL, NULL);

  while (numRecs > histSize) {
    numRecs--;
    DmRemoveRecord(historyDB, (UInt) numRecs);
  }
}
