#include <Pilot.h>
#include "globals.h"
#include "callback.h"
#include "offlineform.h"
#include "send.h"
#include "history.h"
#include "cache.h"
#include "notify.h"
#include "utils.h"
#include "editmenu.h"

static Int editItem;

void OfflineDrawRecord(VoidPtr tableP, Word row, Word column, 
		       RectanglePtr bounds) {
  VoidHand recHandle;
  CharPtr recText;
  Int textLen, width;
  Boolean fits;
  UInt recordNum = 0;
  FontID currFont;
  char * buf;

  CALLBACK_PROLOGUE

  WinEraseRectangle(bounds, 0); 
  recordNum = (UInt) TblGetItemInt (tableP, row, column);

  currFont = FntSetFont (stdFont);
  
  recHandle = DmGetRecord(offlineInfoDB, recordNum);
  buf = (char *) MemHandleLock(recHandle);
  recText = buf + sizeof(ULong) + sizeof(Word) + sizeof(DWord);
  if (StrLen(recText)+1+sizeof(ULong)+sizeof(Word)+sizeof(DWord)
      < MemHandleSize(recHandle)) {
    recText = (char *) recText + StrLen(recText)+1;
  }
  width = bounds->extent.x - 2;
  textLen = StrLen(recText);
  FntCharsInWidth(recText, &width, &textLen, &fits);
  
  WinDrawChars(recText, textLen, bounds->topLeft.x, bounds->topLeft.y);
  
  MemHandleUnlock(recHandle);
  
  DmReleaseRecord(offlineInfoDB, recordNum, false);
  
  FntSetFont (currFont);

  CALLBACK_EPILOGUE
}

Int OfflineLoadTable(FormPtr frm, Int top)  {
  Word row;
  Word lastRecordNum;
  Word numRows;
  VoidHand recordH;
  TablePtr tableP;
  ULong numRecs;
  LocalID lid;
  UInt cardno;
  Int recordNum;
  

  tableP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, OLCacheTable));

  numRows = TblGetNumberOfRows (tableP);

  TblUnhighlightSelection(tableP);

  DmOpenDatabaseInfo(offlineDB, &lid, NULL, NULL, &cardno, NULL);
  DmDatabaseSize(cardno, lid, &numRecs, NULL, NULL);

  if ((top+numRows) > numRecs) {
    top = numRecs - numRows;
  }
  if (top < 0) {
    top = 0;
  }

  recordNum = top;

  for (row = 0; row < numRows; row++, recordNum++) {
    recordH = DmQueryRecord (offlineDB, recordNum);
    
    if (recordH) {
      TblSetItemStyle (tableP, row, 0, customTableItem);      
      TblSetItemInt (tableP, row, 0, (Word) recordNum);
      TblSetRowUsable (tableP, row, true);
      lastRecordNum = recordNum;
    } else {
      TblSetRowUsable (tableP, row, false);
    }
    TblMarkRowInvalid (tableP, row); 
  }
  return top;
}

void OfflineTableLoad(FormPtr frm) {
  TablePtr tableP;

  tableP = FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, OLCacheTable));
  curTop = OfflineLoadTable (frm, 0);
  TblSetCustomDrawProcedure (tableP, 0, OfflineDrawRecord);
  TblSetColumnUsable (tableP, 0, true);	
}

Boolean OfflineCacheHandleEvent (EventPtr e) {

  FormPtr frm;
  Boolean handled=false;

  CALLBACK_PROLOGUE

  switch (e->eType) {
  case winEnterEvent:
    frm = FrmGetActiveForm();  
    FrmDrawForm(frm);
    handled = true;
    break;    
  case frmOpenEvent:   
    frm = FrmGetActiveForm();  
    OfflineTableLoad(frm);
    FrmDrawForm(frm);
    handled = true;
    break;

  case keyDownEvent:
    frm = FrmGetActiveForm();  
    if (e->data.keyDown.chr == pageUpChr) {	
      if (curTop) {
	curTop = OfflineLoadTable (frm, curTop-2);      
	TblDrawTable( FrmGetObjectPtr 
		      (frm, FrmGetObjectIndex (frm, OLCacheTable)));
      }
      handled = true;
    }
    else if (e->data.keyDown.chr == pageDownChr) {	
      curTop = OfflineLoadTable (frm, curTop+2);      
      TblDrawTable( FrmGetObjectPtr 
		    (frm, FrmGetObjectIndex (frm, OLCacheTable)));
      handled = true;
    }
    break;
  case ctlSelectEvent:
    frm = FrmGetActiveForm();  
    switch(e->data.ctlEnter.controlID) {
      Word row, col;
      TablePtr tblP; 
      VoidHand loc;
      void * src,* dst;
      void * buf;

    case OLCacheGoto:  
      tblP =  FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, OLCacheTable));
      if (TblGetSelection(tblP, &row, &col)) {
	loc = DmQueryRecord(offlineInfoDB, TblGetItemInt(tblP, row, col));
	src = MemHandleLock(loc);
	curURLLength = * ((Word *) (src + sizeof(ULong)));
	if (curURL) {
	  MemHandleFree(curURL);
	}
	curURL = MemHandleNew((ULong) curURLLength);
	dst = MemHandleLock(curURL);
	MemMove(dst, src + sizeof(ULong) + sizeof(Word) + sizeof(ULong), curURLLength);
	MemHandleUnlock(loc);
	MemHandleUnlock(curURL);
	SendURL(&conID, curURL, true, true, false, false);
      }
    case OLCacheCancel:
      FrmGotoForm(MainForm);
      handled = true;
      break;
    case OLCacheEdit:
      tblP =  FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, OLCacheTable));
      if (TblGetSelection(tblP, &row, &col)) {	
	editItem = TblGetItemInt(tblP, row, col);
	FrmPopupForm(EditOLCacheForm);
      }
      handled = true;
      break;
    case OLCacheDelete:
      tblP =  FrmGetObjectPtr (frm, FrmGetObjectIndex (frm, OLCacheTable));
      if (TblGetSelection(tblP, &row, &col)) {
	char * url;
	Int size;
	VoidHand URL;

	URL = DmQueryRecord(offlineInfoDB, TblGetItemInt(tblP, row, col));
	buf = MemHandleLock(URL);
	size = * ((Word *) (buf + sizeof(ULong)));
	url = (char *) MemPtrNew(size);
	MemMove(url, (buf + sizeof(ULong) + sizeof(Word) 
		      + sizeof(ULong)), size);
	MemHandleUnlock(URL);
	RemoveFromCache(url, size, offlineInfoDB, offlineDB);
	MemPtrFree(url);
	OfflineLoadTable(frm, curTop);
	TblUnhighlightSelection(tblP);
	TblDrawTable(tblP);
      }
      handled = true;
      break;
    }
  case ctlEnterEvent:
  case ctlRepeatEvent:
    frm = FrmGetActiveForm();  
    switch(e->data.ctlEnter.controlID) {
    case OLCacheUp:
      if (curTop) {
	curTop = OfflineLoadTable (frm, curTop-2);      
	TblDrawTable( FrmGetObjectPtr 
		      (frm, FrmGetObjectIndex (frm, OLCacheTable)));
      }
      handled = true;
      break;
    case OLCacheDown:     
      curTop = OfflineLoadTable (frm, curTop+2);
      TblDrawTable( FrmGetObjectPtr
		    (frm, FrmGetObjectIndex (frm, OLCacheTable)));
      handled = true;
      break;
    }
    break;

  default:
    break;
  }

  CALLBACK_EPILOGUE

  return handled;
} 

Boolean EditOLCacheHandleEvent(EventPtr e) {
  FormPtr frm;
  Boolean handled=false;
  char * url;
  char * name; 
  char * buf;
  VoidHand rec;
  ULong id;
  Word urllen;
  DWord expr;

  CALLBACK_PROLOGUE

  switch (e->eType) {
  case frmOpenEvent:   
    frm = FrmGetActiveForm(); 
    rec = DmQueryRecord(offlineInfoDB, editItem);
    url = MemHandleLock(rec)+sizeof(ULong)+sizeof(Word)+sizeof(DWord);
    if (StrLen(url)+1+sizeof(ULong)+sizeof(Word)+sizeof(DWord)
	< MemHandleSize(rec)) {
      name = url + StrLen(url)+1;
    } else {
      name = 0;
    }
    frm = FrmGetActiveForm();
    if (name) {
      SetFld(FrmGetObjectPtr(frm, FrmGetObjectIndex (frm, EditOLCacheNameF)), 
	     name, StrLen(name));
    }
    SetFld(FrmGetObjectPtr(frm, FrmGetObjectIndex (frm, EditOLCacheURLF)), 
	   url, StrLen(url));
    MemHandleUnlock(rec); 
    FrmDrawForm(frm);
    handled = true;
    break;
  case ctlSelectEvent:
    frm = FrmGetActiveForm();  
    switch(e->data.ctlEnter.controlID) {
    case EditOLCacheSave:
      name = FldGetTextPtr(FrmGetObjectPtr(frm, 
					FrmGetObjectIndex (frm, 
							   EditOLCacheNameF)));
      url = FldGetTextPtr(FrmGetObjectPtr(frm, 
					FrmGetObjectIndex (frm, 
							   EditOLCacheURLF)));
      rec = DmQueryRecord(offlineInfoDB, editItem);
      buf = MemHandleLock(rec);
      id = * ((ULong *) (buf));
      urllen = * ((Word *) (buf+sizeof(ULong)));
      expr = * ((DWord *) (buf+sizeof(ULong)+sizeof(Word)));
      MemHandleUnlock(rec);
      if (name == NULL || StrLen(name) == 0) {
	DmResizeRecord(offlineInfoDB, editItem, 
		       StrLen(url)+1+sizeof(ULong)+sizeof(Word)+sizeof(DWord));
	rec = DmQueryRecord(offlineInfoDB, editItem);
	buf = MemHandleLock(rec);
	DmWrite(buf, sizeof(ULong)+sizeof(Word)+sizeof(DWord), 
		url, StrLen(url)+1);
      } else {
	DmResizeRecord(offlineInfoDB, editItem, 
		       StrLen(url)+StrLen(name)+2+
		       sizeof(ULong)+sizeof(Word)+sizeof(DWord));
	rec = DmQueryRecord(offlineInfoDB, editItem);
	buf = MemHandleLock(rec);
	DmWrite(buf, sizeof(ULong)+sizeof(Word)+sizeof(DWord), 
		url, StrLen(url)+1);
	DmWrite(buf, StrLen(url)+1+sizeof(ULong)+sizeof(Word)+sizeof(DWord),
		name, StrLen(name)+1);	
      }
      DmWrite(buf, 0, &id, sizeof(ULong));
      DmWrite(buf, sizeof(ULong), &urllen, sizeof(Word));
      DmWrite(buf, sizeof(ULong)+sizeof(Word), &expr, sizeof(DWord));
      MemHandleUnlock(rec);
    case EditOLCacheCancel:
      FrmReturnToForm(0);
    }
    handled = true;
    break;
  case menuEvent:
    MenuEraseStatus(NULL);
    if (EditMenuHandleEvent(e->data.menu.itemID, FrmGetActiveForm())) {
	handled = true;
    } else {
	switch(e->data.menu.itemID) {
	}
    }
    handled = true;
    break;

  default:
    break;
  }    

  CALLBACK_EPILOGUE
    
  return handled;
}

